@@ -19,24 +19,30 @@
 #include "namedb.h"
 
 static int write_db (namedb_type *db);
+static int write_number(struct namedb *db, uint32_t number);
 
 struct namedb *
 namedb_new (const char *filename)
 {
 	namedb_type *db;
-	region_type *region = region_create(xalloc, free);
+	region_type *region = region_create_custom(xalloc, free, 
+		DEFAULT_CHUNK_SIZE, DEFAULT_LARGE_OBJECT_SIZE, 
+		DEFAULT_INITIAL_CLEANUP_SIZE, 1);
 	
 	/* Make a new structure... */
 	db = (namedb_type *) region_alloc(region, sizeof(namedb_type));
 	db->region = region;
 	db->domains = domain_table_create(region);
 	db->zones = NULL;
+	db->zone_count = 0;
 	db->filename = region_strdup(region, filename);
+	db->crc = 0xffffffff;
+	db->diff_skip = 0;
+	db->fd = NULL;
 
 	/*
 	 * Unlink the old database, if it exists.  This is useful to
-	 * ensure that NSD (when using mmap) doesn't see the changes
-	 * until a reload is done.
+	 * ensure that NSD doesn't see the changes until a reload is done.
 	 */
 	if (unlink(db->filename) == -1 && errno != ENOENT) {
 		region_destroy(region);
@@ -44,13 +50,12 @@ namedb_new (const char *filename)
 	}
 	
 	/* Create the database */
-        if ((db->fd = fopen(db->filename, "w")) == NULL) {
+    if ((db->fd = fopen(db->filename, "w")) == NULL) {
 		region_destroy(region);
 		return NULL;
 	}
 
-
-	if (!write_data(db->fd, NAMEDB_MAGIC, NAMEDB_MAGIC_SIZE)) {
+	if (!write_data_crc(db->fd, NAMEDB_MAGIC, NAMEDB_MAGIC_SIZE, &db->crc)) {
 		fclose(db->fd);
 		namedb_discard(db);
 		return NULL;
@@ -66,9 +71,15 @@ namedb_save (struct namedb *db)
 	if (write_db(db) != 0) {
 		return -1;
 	}		
+
+	/* Finish up and write the crc */
+	if (!write_number(db, ~db->crc)) {
+		fclose(db->fd);
+		return -1;
+	}
 	
 	/* Write the magic... */
-	if (!write_data(db->fd, NAMEDB_MAGIC, NAMEDB_MAGIC_SIZE)) {
+	if (!write_data_crc(db->fd, NAMEDB_MAGIC, NAMEDB_MAGIC_SIZE, &db->crc)) {
 		fclose(db->fd);
 		return -1;
 	}
@@ -88,16 +99,15 @@ namedb_discard (struct namedb *db)
 	region_destroy(db->region);
 }
 
-
 static int
 write_dname(struct namedb *db, domain_type *domain)
 {
 	const dname_type *dname = domain_dname(domain);
 	
-	if (!write_data(db->fd, &dname->name_size, sizeof(dname->name_size)))
+	if (!write_data_crc(db->fd, &dname->name_size, sizeof(dname->name_size), &db->crc))
 		return 0;
 
-	if (!write_data(db->fd, dname_name(dname), dname->name_size))
+	if (!write_data_crc(db->fd, dname_name(dname), dname->name_size, &db->crc))
 		return 0;
 
 	return 1;
@@ -107,7 +117,7 @@ static int
 write_number(struct namedb *db, uint32_t number)
 {
 	number = htonl(number);
-	return write_data(db->fd, &number, sizeof(number));
+	return write_data_crc(db->fd, &number, sizeof(number), &db->crc);
 }
 
 static int
@@ -131,14 +141,14 @@ write_rrset(struct namedb *db, domain_type *domain, rrset_type *rrset)
 		return 0;
 	
 	type = htons(rrset_rrtype(rrset));
-	if (!write_data(db->fd, &type, sizeof(type)))
+	if (!write_data_crc(db->fd, &type, sizeof(type), &db->crc))
 		return 0;
 
 	klass = htons(rrset_rrclass(rrset));
-	if (!write_data(db->fd, &klass, sizeof(klass)))
+	if (!write_data_crc(db->fd, &klass, sizeof(klass), &db->crc))
 		return 0;
 
-	if (!write_data(db->fd, &rr_count, sizeof(rr_count)))
+	if (!write_data_crc(db->fd, &rr_count, sizeof(rr_count), &db->crc))
 		return 0;
 		
 	for (i = 0; i < rrset->rr_count; ++i) {
@@ -147,11 +157,11 @@ write_rrset(struct namedb *db, domain_type *domain, rrset_type *rrset)
 		uint16_t rdata_count;
 		
 		rdata_count = htons(rr->rdata_count);
-		if (!write_data(db->fd, &rdata_count, sizeof(rdata_count)))
+		if (!write_data_crc(db->fd, &rdata_count, sizeof(rdata_count), &db->crc))
 			return 0;
 
 		ttl = htonl(rr->ttl);
-		if (!write_data(db->fd, &ttl, sizeof(ttl)))
+		if (!write_data_crc(db->fd, &ttl, sizeof(ttl), &db->crc))
 			return 0;
 
 		for (j = 0; j < rr->rdata_count; ++j) {
@@ -161,11 +171,11 @@ write_rrset(struct namedb *db, domain_type *domain, rrset_type *rrset)
 					return 0;
 			} else {
 				uint16_t size = htons(rdata_atom_size(atom));
-				if (!write_data(db->fd, &size, sizeof(size)))
+				if (!write_data_crc(db->fd, &size, sizeof(size), &db->crc))
 					return 0;
-				if (!write_data(db->fd,
+				if (!write_data_crc(db->fd,
 						rdata_atom_data(atom),
-						rdata_atom_size(atom)))
+						rdata_atom_size(atom), &db->crc))
 					return 0;
 			}
 		}
@@ -245,12 +255,12 @@ write_db(namedb_type *db)
 		return -1;
 
 	DEBUG(DEBUG_ZONEC, 1,
-	      (stderr, "Storing %lu domain names\n", (unsigned long) dname_count));
+	      (LOG_INFO, "Storing %lu domain names\n", (unsigned long) dname_count));
 	
 	domain_table_iterate(db->domains, write_dname_iterator, db);
 		   
 	domain_table_iterate(db->domains, write_domain_iterator, db);
-	if (!write_data(db->fd, &terminator, sizeof(terminator)))
+	if (!write_data_crc(db->fd, &terminator, sizeof(terminator), &db->crc))
 		return -1;
 
 	return 0;
