@@ -83,6 +83,7 @@ usage (void)
 		);
 	fprintf(stderr,
 		"  -u user         Change effective uid to the specified user.\n"
+		"  -V level        Specify verbosity level.\n"
 		"  -v              Print version information.\n"
 		);
 	fprintf(stderr, "Version %s. Report bugs to <%s>.\n", 
@@ -368,7 +369,7 @@ main (int argc, char *argv[])
 
 
 	/* Parse the command line... */
-	while ((c = getopt(argc, argv, "46a:c:df:hi:I:l:N:n:P:p:s:u:t:X:v"
+	while ((c = getopt(argc, argv, "46a:c:df:hi:I:l:N:n:P:p:s:u:t:X:V:v"
 #ifndef NDEBUG	
 		"F:L:"
 #endif /* NDEBUG */
@@ -473,6 +474,9 @@ main (int argc, char *argv[])
 		case 'u':
 			nsd.username = optarg;
 			break;
+		case 'V':
+			verbosity = atoi(optarg);
+			break;
 		case 'v':
 			version();
 			/* version exits */
@@ -503,11 +507,11 @@ main (int argc, char *argv[])
 		      (unsigned) strlen(nsd.identity));
 	}
 
-        /* Read options */
-        nsd.options = nsd_options_create(region_create(xalloc, free));
-        if(!parse_options_file(nsd.options, configfile)) {
-                error("nsd: could not read config: %s\n", configfile);
-        }
+    /* Read options */
+	nsd.options = nsd_options_create(region_create(xalloc, free));
+	if(!parse_options_file(nsd.options, configfile)) {
+		error("nsd: could not read config: %s\n", configfile);
+	}
 	if(nsd.options->ip4_only) {
 		for (i = 0; i < MAX_INTERFACES; ++i) {
 			hints[i].ai_family = AF_INET;
@@ -534,6 +538,12 @@ main (int argc, char *argv[])
 			ip = ip->next;
 		}
 	}
+	if (verbosity == 0)
+		verbosity = nsd.options->verbosity;
+#ifndef NDEBUG
+	if (nsd_debug_level > 0 && verbosity == 0)
+		verbosity = nsd_debug_level;
+#endif /* NDEBUG */
 	if(nsd.options->debug_mode) nsd.debug=1;
 	if(!nsd.dbfile)
 	{
@@ -585,6 +595,8 @@ main (int argc, char *argv[])
 			error("cannot chdir to '%s': %s", 
 				nsd.options->zonesdir, strerror(errno));
 		}
+		DEBUG(DEBUG_IPC,1, (LOG_INFO, "changed directory to %s", 
+			nsd.options->zonesdir));
 	}
 	/* get it from the config file */
 #ifdef NSID
@@ -724,6 +736,15 @@ main (int argc, char *argv[])
 	if (nsd.chrootdir) {
 		int l = strlen(nsd.chrootdir);
 
+		/* existing chrootdir: append trailing slash for strncmp checking */
+		if (l>0 && strncmp(nsd.chrootdir + (l-1), "/", 1) != 0) {
+			char *chroot_slash = region_alloc(nsd.region, sizeof(char)*(l+2)); 
+			memcpy(chroot_slash, nsd.chrootdir, sizeof(char)*(l+1)); 
+			strncat(chroot_slash, "/", 1);
+			nsd.chrootdir = chroot_slash; 
+			++l;
+		}
+
 		if (strncmp(nsd.chrootdir, nsd.pidfile, l) != 0) {
 			log_msg(LOG_ERR, "%s is not relative to %s: will not chroot",
 				nsd.pidfile, nsd.chrootdir);
@@ -762,9 +783,7 @@ main (int argc, char *argv[])
 	}
 
 	/* Unless we're debugging, fork... */
-	if (nsd.debug) {
-		nsd.server_kind = NSD_SERVER_BOTH;
-	} else {
+	if (!nsd.debug) {
 		int fd;
 		
 		/* Take off... */
@@ -808,7 +827,6 @@ main (int argc, char *argv[])
 	action.sa_handler = SIG_IGN;
 	sigaction(SIGPIPE, &action, NULL);
 
-
 	/* Get our process id */
 	nsd.pid = getpid();
 
