@@ -439,10 +439,11 @@ answer_notify (struct nsd* nsd, struct query *query)
 		pos = buffer_position(query->packet);
 		buffer_clear(query->packet);
 		buffer_set_position(query->packet, pos);
+		VERBOSITY(2, (LOG_INFO, "Notify received and accepted, forward to xfrd")); 
 		/* tsig is added in add_additional later (if needed) */
 		return QUERY_PROCESSED;
 	}
-	DEBUG(DEBUG_XFRD,1, (LOG_INFO, "got notify %s refused acl: %s %s",
+	VERBOSITY(1, (LOG_INFO, "got notify for zone: %s; Refused by acl: %s %s",
 			dname_to_string(query->qname, NULL),
 			why?why->key_name:"no acl matches", 
 			why?why->ip_address_spec:"."));
@@ -926,6 +927,14 @@ answer_authoritative(struct nsd   *nsd,
 	domain_type *original = closest_match;
 	rrset_type *rrset;
 	
+#ifdef NSEC3
+	if(exact && domain_has_only_NSEC3(closest_match, q->zone)) {
+		exact = 0; /* pretend it does not exist */
+		if(closest_encloser->parent)
+			closest_encloser = closest_encloser->parent;
+	}
+#endif /* NSEC3 */
+
 	if (exact) {
 		match = closest_match;
 	} else if ((rrset=domain_find_rrset(closest_encloser, q->zone, TYPE_DNAME))) {
@@ -990,12 +999,16 @@ answer_authoritative(struct nsd   *nsd,
 		match->rrsets = wildcard_child->rrsets;
 		match->is_existing = wildcard_child->is_existing;
 #ifdef NSEC3
-		match->nsec3_exact = wildcard_child->nsec3_exact;
+		match->nsec3_is_exact = wildcard_child->nsec3_is_exact;
 		match->nsec3_cover = wildcard_child->nsec3_cover;
 		match->nsec3_wcard_child_cover = wildcard_child->nsec3_wcard_child_cover;
-		match->nsec3_ds_parent_exact = wildcard_child->nsec3_ds_parent_exact;
+		match->nsec3_ds_parent_is_exact = wildcard_child->nsec3_ds_parent_is_exact;
 		match->nsec3_ds_parent_cover = wildcard_child->nsec3_ds_parent_cover;
-		nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
+
+		if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
+			/* Only add nsec3 wildcard data when do bit is set */
+			nsec3_answer_wildcard(q, answer, wildcard_child, nsd->db, qname);
+		}
 #endif
 
 		/*
@@ -1011,11 +1024,6 @@ answer_authoritative(struct nsd   *nsd,
 
 	/* Authorative zone.  */
 #ifdef NSEC3
-	if((q->qtype==TYPE_NSEC3 || q->qtype==TYPE_RRSIG || q->qtype==TYPE_ANY)
-		&& match && 
-		domain_has_only_NSEC3(match, q->zone)) {
-		match = 0; /* pretend it does not exist */
-	}
 	if (q->edns.dnssec_ok && q->zone->nsec3_soa_rr) {
 		nsec3_answer_authoritative(&match, q, answer, 
 			closest_encloser, nsd->db, qname);
