@@ -326,7 +326,7 @@ initialize_dname_compression_tables(struct nsd *nsd)
 
 /*
  * Initialize the server, create and bind the sockets.
- * Drop the priviledges and chroot if requested.
+ * Drop the privileges and chroot if requested.
  *
  */
 int
@@ -426,7 +426,6 @@ server_init(struct nsd *nsd)
 #ifdef	SO_REUSEADDR
 		if (setsockopt(nsd->tcp[i].s, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on)) < 0) {
 			log_msg(LOG_ERR, "setsockopt(..., SO_REUSEADDR, ...) failed: %s", strerror(errno));
-			return -1;
 		}
 #endif /* SO_REUSEADDR */
 
@@ -463,6 +462,9 @@ server_init(struct nsd *nsd)
 	if (nsd->chrootdir) {
 		int l = strlen(nsd->chrootdir);
 
+		while (l>0 && nsd->chrootdir[l-1] == '/')
+			--l;
+
 		nsd->dbfile += l;
 		nsd->pidfile += l;
 		nsd->options->xfrdfile += l;
@@ -472,12 +474,15 @@ server_init(struct nsd *nsd)
 			log_msg(LOG_ERR, "unable to chroot: %s", strerror(errno));
 			return -1;
 		}
+		DEBUG(DEBUG_IPC,1, (LOG_INFO, "changed root directory to %s", 
+			nsd->chrootdir));
+
 	}
 #endif
 
 	/* Drop the permissions */
 	if (setgid(nsd->gid) != 0 || setuid(nsd->uid) !=0) {
-		log_msg(LOG_ERR, "unable to drop user priviledges: %s", strerror(errno));
+		log_msg(LOG_ERR, "unable to drop user privileges: %s", strerror(errno));
 		return -1;
 	}
 
@@ -1679,7 +1684,9 @@ handle_tcp_accept(netio_type *netio,
 		   of saying that the client has closed the connection. */
 		if (	errno != EINTR 
 			&& errno != EWOULDBLOCK 
+#ifdef ECONNABORTED
 			&& errno != ECONNABORTED 
+#endif /* ECONNABORTED */
 #ifdef EPROTO
 			&& errno != EPROTO
 #endif /* EPROTO */
