@@ -39,6 +39,7 @@
 #include "util.h"
 #include "zparser.h"
 #include "options.h"
+#include "nsec3.h"
 
 const dname_type *error_dname;
 domain_type *error_domain;
@@ -253,7 +254,7 @@ zparser_conv_period(region_type *region, const char *periodstr)
 
 	/* Allocate required space... */
 	period = (uint32_t) strtottl(periodstr, &end);
-	if (*end != 0) {
+	if (*end != '\0') {
 		zc_error_prev_line("time period is expected");
 	} else {
 		period = htonl(period);
@@ -270,7 +271,7 @@ zparser_conv_short(region_type *region, const char *text)
 	char *end;
    
 	value = htons((uint16_t) strtol(text, &end, 10));
-	if (*end != 0) {
+	if (*end != '\0') {
 		zc_error_prev_line("integer value is expected");
 	} else {
 		r = alloc_rdata_init(region, &value, sizeof(value));
@@ -328,7 +329,7 @@ zparser_conv_certificate_type(region_type *region, const char *text)
 	} else {
 		char *end;
 		id = htons((uint16_t) strtol(text, &end, 10));
-		if (end != 0) {
+		if (*end != '\0') {
 			zc_error_prev_line("certificate type is expected");
 			return NULL;
 		}
@@ -1253,6 +1254,10 @@ check_dname(namedb_type* db)
 		if(domain->is_existing) {
 			/* there may not be DNAMEs above it */
 			domain_type* parent = domain->parent;
+#ifdef NSEC3
+			if(domain_has_only_NSEC3(domain, NULL))
+				continue;
+#endif
 			while(parent) {
 				if(domain_find_rrset_any(parent, TYPE_DNAME)) {
 					zc_error("While checking node %s,",
@@ -1299,13 +1304,13 @@ zone_read(const char *name, const char *zonefile, nsd_options_t* nsd_options)
 			zone_options_t* zopt = zone_options_find(nsd_options, dname);
 			if(zopt && zone_is_slave(zopt)) {
 				zc_warning("slave zone %s with no zonefile '%s'(%s) will "
-					"force zone transfer.\n", 
+					"force zone transfer.", 
 					name, zonefile, strerror(errno));
 				return;
 			}
 		}
 		/* cannot happen with stdin - so no fix needed for zonefile */
-		zc_error("cannot open '%s': %s\n", zonefile, strerror(errno));
+		zc_error("cannot open '%s': %s", zonefile, strerror(errno));
 		return;
 	}
 
@@ -1366,7 +1371,7 @@ main (int argc, char **argv)
 {
 	struct namedb *db;
 	char *origin = NULL;
-	int c;
+	int c, errn;
 	region_type *global_region;
 	region_type *rr_region;
 	const char* configfile= CONFIGFILE;
@@ -1445,6 +1450,8 @@ main (int argc, char **argv)
 			fprintf(stderr, "zonec: cannot chdir to %s: %s\n", zonesdir, strerror(errno));
 			exit(1);
 		}
+		else
+			log_msg(LOG_NOTICE, "changed directory to %s\n", zonesdir); 
 	}
 	if(dbfile == 0) {
 		if(nsd_options && nsd_options->database) dbfile = nsd_options->database;
@@ -1453,7 +1460,8 @@ main (int argc, char **argv)
 
 	/* Create the database */
 	if ((db = namedb_new(dbfile)) == NULL) {
-		fprintf(stderr, "zonec: error creating the database: %s\n", dbfile);
+		fprintf(stderr, "zonec: error creating the database (%s): %s\n", 
+			dbfile, strerror(errno));
 		exit(1);
 	}
 
@@ -1513,7 +1521,7 @@ main (int argc, char **argv)
 	
 	/* Close the database */
 	if (namedb_save(db) != 0) {
-		fprintf(stderr, "zonec: error saving the database: %s\n", strerror(errno));
+		fprintf(stderr, "zonec: error writing the database (%s): %s\n", db->filename, strerror(errno));
 		namedb_discard(db);
 		exit(1);
 	}
